#include <tap.h>
#include "libFireBird.h"

TAP_ID( 0x8100fee1 );
#define TAPNAME "Cut Path to File"
TAP_PROGRAM_NAME(TAPNAME);
TAP_AUTHOR_NAME("R2-D2");
TAP_DESCRIPTION("Cuts a section from the playing file");
TAP_ETCINFO(__DATE__);

dword *bkCount = NULL;  // Count of playing file's current bookmarks
dword *bkMarks = NULL;  // The 64 possible bookmarks for the playing file
dword fwCutPath = 0; // Firmware cutting routine

#define TEMPFILE "_CUT_.ts"

#define PLAYPATH (2)

// This cuts the blocks starting at startBlock from the path into filename in the current dir
dword cutPath(dword path, dword startBlock, dword blocks, char *filename)
{
	tTAPTable *t = (void*)TAP_TableAddress;
	dword dir = t[TAP_TableIndex].pActDir;
	return CallFirmwareA(fwCutPath,dir,path,startBlock,blocks,filename);
}

dword TAP_EventHandler( word event, dword param1, dword param2 )
{
	return param1;
}

/*****************************************************************************/
/* TAP_main */
/*****************************************************************************/

int TAP_Main(void)
{
	short *pBook = NULL;
	InitTAPex();
	TAP_Print("Installing...\n");
	pBook = (short*)FindInstructionSequence(
		"3C1981B5 3C1881B5 8F259400 27189404 27060200 0C052999 02C02025",
		"FFFF0000 FFFF0000 FFFF0000 FFFF0000 FFFFFFFF FC000000 FEFFFFFF",
		0x800C0000, 0x801C0000, 0, FALSE);
	fwCutPath = FindInstructionSequence(
		"8FAF0050 AFAF0010 03C03825 02C02025 02A02825 02803025 0C026B57 00000000",
		"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FC000000 FFFFFFFF",
		0x80060000, 0x80160000, 0, TRUE);
	if(pBook == NULL || fwCutPath == 0)
	{
		TAP_Print("Unsupported firmware %d:%x\n",SysID,ApplID);
	}
	else
	{
		dword i, tot;
		bkCount = (dword*)((pBook[1] << 16) + pBook[5]);
		bkMarks = (dword*)((pBook[3] << 16) + pBook[7] + 512);
		TAP_Print("cutPath=%p, bkCount=%p (%d), bkMarks=%p\n",fwCutPath,bkCount,*bkCount,bkMarks);
		if(*bkCount > 1)
		{
			dword res;
			if(TAP_Hdd_Exist(TEMPFILE))
			{
				TAP_Print("Deleting existing cut file\n");
				TAP_Hdd_Delete(TEMPFILE);
			}
			TAP_Print("Cutting out section from %8x to %8x\n",bkMarks[0],bkMarks[1]);
			res = cutPath(PLAYPATH,bkMarks[0],bkMarks[1],TEMPFILE);
			TAP_Print("Cutter returned %x\n",res);
		}
	}
	return 0;
}
